@@ -77,6 +77,9 @@ _barbican_opts = [
                choices=['public', 'internal', 'admin'],
                help='Specifies the type of endpoint.  Allowed values are: '
                     'public, private, and admin'),
+    cfg.StrOpt('barbican_region_name',
+               default=None,
+               help='Specifies the region of the chosen endpoint.'),
 
 ]
 
@@ -188,18 +191,20 @@ class BarbicanKeyManager(key_manager.KeyManager):
             raise exception.Forbidden(reason=msg)
 
     def _get_barbican_endpoint(self, auth, sess):
-        barbican = self.conf.barbican
-        if barbican.barbican_endpoint:
-            return barbican.barbican_endpoint
+        if self.conf.barbican.barbican_endpoint:
+            return self.conf.barbican.barbican_endpoint
         elif getattr(auth, 'service_catalog', None):
             endpoint_data = auth.service_catalog.endpoint_data_for(
-                service_type='key-manager')
+                service_type='key-manager',
+                interface=self.conf.barbican.barbican_endpoint_type,
+                region_name=self.conf.barbican.barbican_region_name)
             return endpoint_data.url
         else:
-            service_parameters = {'service_type': 'key-manager',
-                                  'service_name': 'barbican',
-                                  'interface': barbican.barbican_endpoint_type}
-            return auth.get_endpoint(sess, **service_parameters)
+            return auth.get_endpoint(
+                sess,
+                service_type='key-manager',
+                interface=self.conf.barbican.barbican_endpoint_type,
+                region_name=self.conf.barbican.barbican_region_name)
 
     def _create_base_url(self, auth, sess, endpoint):
         api_version = None
@@ -207,7 +212,9 @@ class BarbicanKeyManager(key_manager.KeyManager):
             api_version = self.conf.barbican.barbican_api_version
         elif getattr(auth, 'service_catalog', None):
             endpoint_data = auth.service_catalog.endpoint_data_for(
-                service_type='key-manager')
+                service_type='key-manager',
+                interface=self.conf.barbican.barbican_endpoint_type,
+                region_name=self.conf.barbican.barbican_region_name)
             api_version = endpoint_data.api_version
         elif getattr(auth, 'get_discovery', None):
             discovery = auth.get_discovery(sess, url=endpoint)
