@@ -17,9 +17,9 @@
 Test cases for the barbican key manager.
 """
 import calendar
+from unittest import mock
 
 from barbicanclient import exceptions as barbican_exceptions
-import mock
 from oslo_utils import timeutils
 
 from castellan.common import exception
@@ -76,6 +76,198 @@ class BarbicanKeyManagerTestCase(test_key_manager.KeyManagerTestCase):
         self.key_mgr._barbican_client = self.mock_barbican
         self.key_mgr._current_context = self.ctxt
 
+    def test_barbican_endpoint(self):
+        endpoint_data = mock.Mock()
+        endpoint_data.url = 'http://localhost:9311'
+
+        auth = mock.Mock(spec=['service_catalog'])
+        auth.service_catalog.endpoint_data_for.return_value = endpoint_data
+
+        endpoint = self.key_mgr._get_barbican_endpoint(auth, mock.Mock())
+        self.assertEqual(endpoint, 'http://localhost:9311')
+        auth.service_catalog.endpoint_data_for.assert_called_once_with(
+            service_type='key-manager', interface='public',
+            region_name=None)
+
+    def test_barbican_endpoint_with_endpoint_type(self):
+        self.key_mgr.conf.barbican.barbican_endpoint_type = 'internal'
+
+        endpoint_data = mock.Mock()
+        endpoint_data.url = 'http://localhost:9311'
+
+        auth = mock.Mock(spec=['service_catalog'])
+        auth.service_catalog.endpoint_data_for.return_value = endpoint_data
+
+        endpoint = self.key_mgr._get_barbican_endpoint(auth, mock.Mock())
+        self.assertEqual(endpoint, 'http://localhost:9311')
+        auth.service_catalog.endpoint_data_for.assert_called_once_with(
+            service_type='key-manager', interface='internal',
+            region_name=None)
+
+    def test_barbican_endpoint_with_region_name(self):
+        self.key_mgr.conf.barbican.barbican_region_name = 'regionOne'
+
+        endpoint_data = mock.Mock()
+        endpoint_data.url = 'http://localhost:9311'
+
+        auth = mock.Mock(spec=['service_catalog'])
+        auth.service_catalog.endpoint_data_for.return_value = endpoint_data
+
+        endpoint = self.key_mgr._get_barbican_endpoint(auth, mock.Mock())
+        self.assertEqual(endpoint, 'http://localhost:9311')
+        auth.service_catalog.endpoint_data_for.assert_called_once_with(
+            service_type='key-manager', interface='public',
+            region_name='regionOne')
+
+    def test_barbican_endpoint_from_config(self):
+        self.key_mgr.conf.barbican.barbican_endpoint = 'http://localhost:9311'
+
+        endpoint = self.key_mgr._get_barbican_endpoint(
+            mock.Mock(), mock.Mock())
+        self.assertEqual(endpoint, 'http://localhost:9311')
+
+    def test_barbican_endpoint_by_get_endpoint(self):
+        auth = mock.Mock(spec=['get_endppint'])
+        sess = mock.Mock()
+        auth.get_endpoint = mock.Mock(return_value='http://localhost:9311')
+
+        endpoint = self.key_mgr._get_barbican_endpoint(auth, sess)
+        self.assertEqual(endpoint, 'http://localhost:9311')
+        auth.get_endpoint.assert_called_once_with(
+            sess, service_type='key-manager', interface='public',
+            region_name=None)
+
+    def test_barbican_endpoint_by_get_endpoint_with_endpoint_type(self):
+        self.key_mgr.conf.barbican.barbican_endpoint_type = 'internal'
+
+        auth = mock.Mock(spec=['get_endppint'])
+        sess = mock.Mock()
+        auth.get_endpoint = mock.Mock(return_value='http://localhost:9311')
+
+        endpoint = self.key_mgr._get_barbican_endpoint(auth, sess)
+        self.assertEqual(endpoint, 'http://localhost:9311')
+        auth.get_endpoint.assert_called_once_with(
+            sess, service_type='key-manager', interface='internal',
+            region_name=None)
+
+    def test_barbican_endpoint_by_get_endpoint_with_region_name(self):
+        self.key_mgr.conf.barbican.barbican_region_name = 'regionOne'
+
+        auth = mock.Mock(spec=['get_endppint'])
+        sess = mock.Mock()
+        auth.get_endpoint = mock.Mock(return_value='http://localhost:9311')
+
+        endpoint = self.key_mgr._get_barbican_endpoint(auth, sess)
+        self.assertEqual(endpoint, 'http://localhost:9311')
+        auth.get_endpoint.assert_called_once_with(
+            sess, service_type='key-manager', interface='public',
+            region_name='regionOne')
+
+    def test_base_url_old_version(self):
+        version = "v1"
+        self.key_mgr.conf.barbican.barbican_api_version = version
+        endpoint = "http://localhost:9311"
+        base_url = self.key_mgr._create_base_url(mock.Mock(),
+                                                 mock.Mock(),
+                                                 endpoint)
+        self.assertEqual(endpoint + "/" + version, base_url)
+
+    def test_base_url_new_version(self):
+        version = "v1"
+        self.key_mgr.conf.barbican.barbican_api_version = version
+        endpoint = "http://localhost/key_manager"
+        base_url = self.key_mgr._create_base_url(mock.Mock(),
+                                                 mock.Mock(),
+                                                 endpoint)
+        self.assertEqual(endpoint + "/" + version, base_url)
+
+    def test_base_url_service_catalog(self):
+        endpoint_data = mock.Mock()
+        endpoint_data.api_version = 'v321'
+
+        auth = mock.Mock(spec=['service_catalog'])
+        auth.service_catalog.endpoint_data_for.return_value = endpoint_data
+
+        endpoint = "http://localhost/key_manager"
+
+        base_url = self.key_mgr._create_base_url(auth,
+                                                 mock.Mock(),
+                                                 endpoint)
+        self.assertEqual(endpoint + "/" + endpoint_data.api_version, base_url)
+        auth.service_catalog.endpoint_data_for.assert_called_once_with(
+            service_type='key-manager', interface='public',
+            region_name=None)
+
+    def test_base_url_service_catalog_with_endpoint_type(self):
+        self.key_mgr.conf.barbican.barbican_endpoint_type = 'internal'
+
+        endpoint_data = mock.Mock()
+        endpoint_data.api_version = 'v321'
+
+        auth = mock.Mock(spec=['service_catalog'])
+        auth.service_catalog.endpoint_data_for.return_value = endpoint_data
+
+        endpoint = "http://localhost/key_manager"
+
+        base_url = self.key_mgr._create_base_url(auth,
+                                                 mock.Mock(),
+                                                 endpoint)
+        self.assertEqual(endpoint + "/" + endpoint_data.api_version, base_url)
+        auth.service_catalog.endpoint_data_for.assert_called_once_with(
+            service_type='key-manager', interface='internal',
+            region_name=None)
+
+    def test_base_url_service_catalog_with_region_name(self):
+        self.key_mgr.conf.barbican.barbican_region_name = 'regionOne'
+
+        endpoint_data = mock.Mock()
+        endpoint_data.api_version = 'v321'
+
+        auth = mock.Mock(spec=['service_catalog'])
+        auth.service_catalog.endpoint_data_for.return_value = endpoint_data
+
+        endpoint = "http://localhost/key_manager"
+
+        base_url = self.key_mgr._create_base_url(auth,
+                                                 mock.Mock(),
+                                                 endpoint)
+        self.assertEqual(endpoint + "/" + endpoint_data.api_version, base_url)
+        auth.service_catalog.endpoint_data_for.assert_called_once_with(
+            service_type='key-manager', interface='public',
+            region_name='regionOne')
+
+    def test_base_url_raise_exception(self):
+        auth = mock.Mock(spec=['get_discovery'])
+        sess = mock.Mock()
+        discovery = mock.Mock()
+        discovery.raw_version_data = mock.Mock(return_value=[])
+        auth.get_discovery = mock.Mock(return_value=discovery)
+
+        endpoint = "http://localhost/key_manager"
+
+        self.assertRaises(exception.KeyManagerError,
+                          self.key_mgr._create_base_url,
+                          auth, sess, endpoint)
+        auth.get_discovery.assert_called_once_with(sess, url=endpoint)
+        self.assertEqual(1, discovery.raw_version_data.call_count)
+
+    def test_base_url_get_discovery(self):
+        version = 'v100500'
+        auth = mock.Mock(spec=['get_discovery'])
+        sess = mock.Mock()
+        discovery = mock.Mock()
+        auth.get_discovery = mock.Mock(return_value=discovery)
+        discovery.raw_version_data = mock.Mock(return_value=[{'id': version}])
+
+        endpoint = "http://localhost/key_manager"
+
+        base_url = self.key_mgr._create_base_url(auth,
+                                                 sess,
+                                                 endpoint)
+        self.assertEqual(endpoint + "/" + version, base_url)
+        auth.get_discovery.assert_called_once_with(sess, url=endpoint)
+        self.assertEqual(1, discovery.raw_version_data.call_count)
+
     def test_create_key(self):
         # Create order_ref_url and assign return value
         order_ref_url = ("http://localhost:9311/v1/orders/"
@@ -189,6 +381,10 @@ class BarbicanKeyManagerTestCase(test_key_manager.KeyManagerTestCase):
         original_secret_metadata.bit_length = mock.sentinel.bit
         original_secret_metadata.secret_type = 'symmetric'
 
+        key_id = "43ed09c3-e551-4c24-b612-e619abe9b534"
+        key_ref = ("http://localhost:9311/v1/secrets/" + key_id)
+        original_secret_metadata.secret_ref = key_ref
+
         created = timeutils.parse_isotime('2015-10-20 18:51:17+00:00')
         original_secret_metadata.created = created
         created_formatted = timeutils.parse_isotime(str(created))
@@ -204,6 +400,7 @@ class BarbicanKeyManagerTestCase(test_key_manager.KeyManagerTestCase):
         key = self.key_mgr.get(self.ctxt, self.key_id)
 
         self.get.assert_called_once_with(self.secret_ref)
+        self.assertEqual(key_id, key.id)
         self.assertEqual(key_name, key.name)
         self.assertEqual(original_secret_data, key.get_encoded())
         self.assertEqual(created_posix, key.created)
@@ -360,6 +557,10 @@ class BarbicanKeyManagerTestCase(test_key_manager.KeyManagerTestCase):
         original_secret_metadata.bit_length = mock.sentinel.bit
         original_secret_metadata.secret_type = 'symmetric'
 
+        key_id = "43ed09c3-e551-4c24-b612-e619abe9b534"
+        key_ref = ("http://localhost:9311/v1/secrets/" + key_id)
+        original_secret_metadata.secret_ref = key_ref
+
         created = timeutils.parse_isotime('2015-10-20 18:51:17+00:00')
         original_secret_metadata.created = created
         created_formatted = timeutils.parse_isotime(str(created))
@@ -380,6 +581,7 @@ class BarbicanKeyManagerTestCase(test_key_manager.KeyManagerTestCase):
         key = key_list[0]
 
         self.list.assert_called_once()
+        self.assertEqual(key_id, key.id)
         self.assertEqual(key_name, key.name)
         self.assertEqual(original_secret_data, key.get_encoded())
         self.assertEqual(created_posix, key.created)
